/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsProtoUdp.c
    
Abstract:

    Implementation for functions for protocol UDP

--*/

#include "EmsNet.h"
#include "EmsTclInit.h"
#include "EmsPktMain.h"
#include "EmsPktCreate.h"
#include "EmsProtocols.h"
#include "EmsProtoEth.h"
#include "EmsProtoIp.h"
#include "EmsProtoUdp.h"
#include "EmsUtilityString.h"

STATIC
libnet_t          *
UdpCreatePacket (
  IN INT32 Argc,
  IN INT8  *Argv[]
  );

STATIC
FIELD_T           *
UdpUnpackPacket (
  IN UINT8  *Packet,
  IN UINT32 Length
  );

PROTOCOL_ENTRY_T  UdpProtocol = {
  "UDP",            /* name */
  UdpCreatePacket,  /* Create packet routine */
  UdpUnpackPacket,  /* Unpacket routine */
  NULL              /* UDP TCL initialzation routine*/
};

STATIC UINT16     UdpSrcPort;
STATIC UINT16     UdpDstPort;
STATIC UINT16     UdpLength;
STATIC UINT16     UdpChecksum;
STATIC PAYLOAD_T  UdpPayload = { NULL, 0 };

#define DEFAULT_CHECKSUM  0x00

STATIC BOOLEAN    SrcPortOk;
STATIC BOOLEAN    DstPortOk;
STATIC BOOLEAN    LengthOk;
STATIC BOOLEAN    ChecksumOk;
STATIC BOOLEAN    PayloadOk;

#define DEFAULT_SRC_PORT  0x00

FIELD_T           UdpField[] = {
  /* name         type         value           Mandatory Isset?*/
  {
    "Udp_sp",
    OCTET2,
    &UdpSrcPort,
    FALSE,
    &SrcPortOk
  },
  {
    "Udp_dp",
    OCTET2,
    &UdpDstPort,
    TRUE,
    &DstPortOk
  },
  {
    "Udp_len",
    OCTET2,
    &UdpLength,
    FALSE,
    &LengthOk
  },
  {
    "Udp_sum",
    OCTET2,
    &UdpChecksum,
    FALSE,
    &ChecksumOk
  },
  {
    "Udp_payload",
    PAYLOAD,
    &UdpPayload,
    FALSE,
    &PayloadOk
  },
  {
    NULL
  }
};

STATIC
libnet_t *
UdpCreatePacket (
  INT32 Argc,
  INT8  *Argv[]
  )
/*++

Routine Description:

  Create an UDP packet

Arguments:

  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  NULL if error occurs
  The pointer to a libnet_t type's object

--*/
{
  libnet_t      *l;
  libnet_ptag_t Tag;
  UINT32        Index;
  INT8          ErrBuf[LIBNET_ERRBUF_SIZE];
  UINT32        Len;
  EMS_STATUS    Status;
  //
  // Initialize all the local variables
  //
  for (Index = 0; UdpField[Index].Name; Index++) {
    *UdpField[Index].IsOk = FALSE;
    Status                = GetValueByName (&UdpField[Index], Argc, Argv);
    if (IsError (Status)) {
      return NULL;
    }
  }
  //
  //  Initialize the libnet
  //
  l = libnet_init (
        LIBNET_LINK,
        /* injection type */EmsInterface,
        /* network interface */ErrBuf /* errbuf */
        );
  if (NULL == l) {
    return NULL;
  }

  /*
 * Build IP packet
 */
  Len = LIBNET_UDP_H;
  Len += PayloadOk ? UdpPayload.Len : 0;

  Tag = libnet_build_udp (
          SrcPortOk ? UdpSrcPort : DEFAULT_SRC_PORT,
          UdpDstPort,
          Len,
          ChecksumOk ? UdpChecksum : DEFAULT_CHECKSUM,
          PayloadOk ? UdpPayload.Payload : NULL,
          PayloadOk ? UdpPayload.Len : 0,
          l,
          0
          );
  if (-1 == Tag) {
    return NULL;
  }

  Len = LIBNET_IPV4_H + LIBNET_UDP_H;
  Len += PayloadOk ? UdpPayload.Len : 0;

  Tag = libnet_build_ipv4 (
          Len,
          0,
          /* TOS */GetIpId (),
          /* IP ID */0,
          /* IP Frag */0x40,
          /* TTL */IPPROTO_UDP,
          /* protocol */0,
          /* checksum */htonl (LocalIp),
          htonl (RemoteIp),
          NULL,
          /* payload */0,
          /* payload size */l,
          0
          );
  if (-1 == Tag) {
    return NULL;
  }

  Tag = libnet_build_ethernet (
          RemoteEthaddr,
          LocalEthaddr,
          ETHERTYPE_IP,
          NULL,
          0,
          l,
          0
          );
  if (-1 == Tag) {
    return NULL;
  }

  return l;
}

STATIC
FIELD_T *
UdpUnpackPacket (
  IN UINT8  *Packet,
  IN UINT32 Length
  )
/*++

Routine Description:

  Unpack an UDP packet

Arguments:

  Packet  - The packet should be unpacked
  Length  - The size of the packet

Returns:

  NULL if error occurs
  The pointer to a FIELD_T type's object if successfully

--*/
{
  UDP_HEADER  *Header;
  UINT32      PayloadLen;

  if (Length < (sizeof (UDP_HEADER) + sizeof (IP_HEADER) + sizeof (ETH_HEADER))) {
    return NULL;
  }

  Header      = (UDP_HEADER *) (Packet + sizeof (ETH_HEADER) + sizeof (IP_HEADER));

  UdpSrcPort  = ntohs (Header->UdpSrcPort);
  UdpDstPort  = ntohs (Header->UdpDstPort);
  UdpLength   = ntohs (Header->UdpLength);
  UdpChecksum = ntohs (Header->UdpChecksum);

  if (NULL != UdpPayload.Payload) {
    free (UdpPayload.Payload);
    UdpPayload.Payload  = NULL;
    UdpPayload.Len      = 0;
  }

  PayloadLen = Length - sizeof (UDP_HEADER) - sizeof (IP_HEADER) - sizeof (ETH_HEADER);
  if (PayloadLen) {
    UdpPayload.Payload = malloc (PayloadLen);
    memcpy (UdpPayload.Payload, Packet + sizeof (UDP_HEADER) + sizeof (IP_HEADER) + sizeof (ETH_HEADER), PayloadLen);
    UdpPayload.Len = PayloadLen;
  }

  return UdpField;
}
