/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCRT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  SCRTApp.c

--*/

#include "SCRTApp.h"
#include "ParseConf.h"

EFI_GUID  gRuntimeFuncAddressGuid = RUNTIME_FUNC_ADDRESS_GUID;

VOID
PrintUsage ( 
  VOID
  )
/*++

Routine Description:

  Print usage information for this utility.
  
Arguments:

  NONE

Returns:

  NONE
  
--*/  
{
  Print(L"SCRTApp Usage:\n");
  Print(L"  -f  FileName  FileName is test case configuration file\n");
  Print(L"  -g  FileName  FileName is test case result/log file\n");
  Print(L"Example:\n");
  Print(L"SCRTApp -f SCRT.conf\n");
  Print(L"SCRTApp -g SCRT.log\n");
}


EFI_STATUS
InitializeSCRT (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  )
/*++

Routine Description:

  Go to virtual mode to test the UEFI Runtime Services.

Arguments:

  ImageHandle  - Handle for the image of this driver
  SystemTable  - Pointer to the EFI System Table

Returns:

  EFI_STATUS

--*/
{
  UINTN                   MemoryMapSize;
  UINTN                   VirtualMapSize;
  UINTN                   MapKey;
  UINTN                   DescriptorSize;
  UINTN                   VirtualFunc;
  UINT32                  DescriptorVersion;
  VOID                    *FileBuffer;
  UINTN                   FileSize;
  UINT32                  ArgIndex;
  CHAR16                  InfFileName[200];
  BOOLEAN                 ConfigFlag;
  BOOLEAN                 GenLogFlag;
  EFI_STATUS              Status;
  MEMORY_FILE             InfMemoryFile;
  EFI_FILE_INFO           *FileInfo;
  CONF_INFO               ConfInfo;
  EFI_FILE_HANDLE         FileHandle;
  EFI_MEMORY_DESCRIPTOR   *MemoryMap;
  EFI_MEMORY_DESCRIPTOR   *TempVirtualMemoryMap;
  EFI_MEMORY_DESCRIPTOR   *VirtualMemoryMap;
  UINTN                   HandOffAddr;
  UINTN                   Index;
  SCRT_STUB_TABLE         *StubTable;
  BOOLEAN                 SctTableFound;
  
  VirtualMapSize        = 0;
  MemoryMapSize         = 0;
  MemoryMap             = NULL;
  FileHandle            = NULL;
  FileInfo              = NULL;
  FileBuffer            = NULL;
  ConfigFlag            = FALSE;
  GenLogFlag            = FALSE;
  SctTableFound         = FALSE;
  HandOffAddr            = 0;

  ConfInfo.InfoData = 0;

  EFI_SHELL_APP_INIT (ImageHandle, SystemTable);
  
  Status = LibFilterNullArgs ();
  if (EFI_ERROR (Status)) {
    return Status;
  }

  //
  // Initialize Configuration File Name
  //
  UnicodeStrCpy (InfFileName, L"");

  //
  // Parse the arguments
  //
  ArgIndex = (UINT32)SI->Argc;

  if ((ArgIndex != 3) || ((UnicodeStrCmp(SI->Argv[1], L"-f")) && (UnicodeStrCmp(SI->Argv[1], L"-g")))) {
    Print(L"SCRTApp: invalid option\n");
    PrintUsage ();
    return EFI_INVALID_PARAMETER;
  } else {
    UnicodeStrCpy (InfFileName, SI->Argv[2]);
    if (UnicodeStrCmp(SI->Argv[1], L"-g") == 0)
      GenLogFlag = TRUE;
    else
      ConfigFlag = TRUE;
  }


  if (GenLogFlag) {
    SCRTLogProcess(InfFileName);
  } else {   

    //
    // Open the Configuration file
    //  
    Status = LibOpenFileByName (
               InfFileName,
               &FileHandle,
               EFI_FILE_MODE_READ,
               0
               );
    if (EFI_ERROR (Status)) {
      Print(L"Fail to open the configuration file\n");     
      return Status;
    }
  
    //
    // If the file is directory, abort
    //
    FileInfo = LibGetFileInfo (FileHandle);
    if (FileInfo == NULL) {
      Print(L"Fail to open the configuration file\n");     
      return EFI_ABORTED;  
    } else if (FileInfo->Attribute & EFI_FILE_DIRECTORY) {
      Print(L"Fail to open the configuration file\n");     
      return EFI_ABORTED;
    }

    //
    // Malloc memory for read file.
    //
    FileBuffer = AllocatePool (CONF_FILE_SIZE);
    if (FileBuffer == NULL) {
      return EFI_OUT_OF_RESOURCES;
    }
  
    FileSize   = CONF_FILE_SIZE;
    Status     = LibReadFile (FileHandle, &FileSize, FileBuffer);
    if (!EFI_ERROR (Status) && (FileSize == 0)) {
      return EFI_NOT_FOUND; 
    }
  
    //
    // Initialize info file structures
    //
    InfMemoryFile.FileImage           = FileBuffer;
    InfMemoryFile.CurrentFilePointer  = FileBuffer;
    InfMemoryFile.Eof                 = (VOID*)((UINTN)FileBuffer + FileSize);
  
    //
    // Parse the FV inf file for header information
    //
    ParseConf (&InfMemoryFile, &ConfInfo);
    //
    // Dont't use FileBuffer any more, Free it.
    //
    FreePool (FileBuffer);
  

    for (Index = 0; Index < ST->NumberOfTableEntries; Index++) {
      if (EfiCompareGuid (&gRuntimeFuncAddressGuid, &(ST->ConfigurationTable[Index].VendorGuid))) {
        StubTable = ST->ConfigurationTable[Index].VendorTable;
        if (StubTable->Signature != SCRT_STUB_TABLE_SIGNATURE) {
          Print(L"Find Corrupted SCRT Table!\n");
          EFI_DEADLOOP();
        } else {
          SctTableFound =TRUE;
          VirtualFunc = (UINTN)StubTable->FuncAddr;
          Print(L"Find RuntimeFunc @ 0x%x registered in SCRT Table\n",VirtualFunc);        
        }
      }
    }

    if (!SctTableFound) {
      Print(L"Fail to find SCRT Table!\n");
      EFI_DEADLOOP();
    }
  
    //
    // Change RuntimeTestFunc physical address to virtual address in advance. Each Arch, IA32, X64 
    // and IPF has its own strategy.
    //
    ConvertRuntimeFuncPtr(&VirtualFunc);
  
    //
    // Allocate memory prior to GetMemoryMap. IA32/X64 needs memory for pagetables, and IPF needs for IVA.
    //
    DoMemoryAllocation(VirtualFunc);
  
    //
    // Prepare runtime hand-off information.
    //
    RuntimeHandOff(&ConfInfo, &HandOffAddr);
  
    //
    // First call GetMemoryMap() to query the current System MemoryMap Size.
    //
    BS->GetMemoryMap (
          &MemoryMapSize,
          MemoryMap,
          &MapKey,
          (VOID *) &DescriptorSize,
          (VOID *) &DescriptorVersion
          );
    //
    // The memory size needs add a constant, because there are two AllocatePool()
    // before the second GetMemoryMap(), the action of allocate pool will increase
    // the MemoryMapSize. The constant of 1024 is enough.
    //
    MemoryMapSize += 1024;
  
    Status = BS->AllocatePool (
                  EfiRuntimeServicesData,
                  MemoryMapSize,
                  &MemoryMap
                  );
    if (EFI_ERROR (Status)) {
      return Status;
    }
  
    Status = BS->AllocatePool (
                  EfiRuntimeServicesData,
                  MemoryMapSize,
                  &VirtualMemoryMap
                  );
    if (EFI_ERROR (Status)) {
      return Status;
    }

    Status = BS->GetMemoryMap (
                  &MemoryMapSize,
                  MemoryMap,
                  &MapKey,
                  (VOID *) &DescriptorSize,
                  (VOID *) &DescriptorVersion
                  );
    if (EFI_ERROR (Status)) {
      return Status;
    }
  
    TempVirtualMemoryMap = VirtualMemoryMap;
    ZeroMem ((VOID *) VirtualMemoryMap, MemoryMapSize);
  
    //
    // Change Runtime Usage MemoryMap entry from physical address to virtual address.
    // Each Arch, IA32, X64 and IPF has its own strategy. 
    //
    PrepareVirtualAddressMap (MemoryMapSize,DescriptorSize,MemoryMap,TempVirtualMemoryMap, &VirtualMapSize);
  
    //
    // Call ExitBootServices to terminate all boot services.
    //
    Status = BS->ExitBootServices (ImageHandle, MapKey);
  
    if (EFI_ERROR (Status)) {
      return Status;
    }

    //
    // Call VirtualAddressMap to change the address of the runtime components of the firmware 
    // to the new virtual address.
    //
    Status = RT->SetVirtualAddressMap (
                  VirtualMapSize,
                  DescriptorSize,
                  DescriptorVersion,
                  VirtualMemoryMap
                  );
  
    if (EFI_ERROR (Status)) {
      return Status;
    }
  
    //
    // Enable Virtual addressing mode to invoke Runtime Test Function.
    //
    JumpVirtualMode(VirtualFunc, HandOffAddr);
  
    //
    // Should never be here
    //

  }

  return EFI_SUCCESS;
}
