/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  SctDebug.c

Abstract:

  This file provides the debug macro services for SCT.

--*/

#include "Sct.h"

//
// Module variables
//

EFI_FILE *mLogFile = NULL;

//
// External functions implementation
//

EFI_STATUS
InitializeDebugServices (
  VOID
  )
/*++

Routine Description:

  Initialize the debug services.

Returns:

  EFI_SUCCESS   - Successful.
  Other value   - Something failed.

--*/
{
  EFI_STATUS  Status;
  CHAR16      *FileName;

  //
  // Check the pre-conditions
  //
  if ((gFT == NULL) || (gFT->TolProtocol == NULL)) {
    return EFI_NOT_READY;
  }

  //
  // Open the system log file
  //
  FileName = PoolPrint (L"%s\\%s", gFT->FilePath, EFI_SCT_FILE_LOG);
  if (FileName == NULL) {
    return EFI_OUT_OF_RESOURCES;
  }

  Status = gFT->TolProtocol->Open (
                               gFT->TolProtocol,
                               gFT->DevicePath,
                               FileName,
                               TRUE,                  // Overwrite
                               &mLogFile
                               );
  if (EFI_ERROR(Status)) {
    BS->FreePool (FileName);
    return Status;
  }

  BS->FreePool (FileName);

  //
  // Done
  //
  return EFI_SUCCESS;
}


EFI_STATUS
FreeDebugServices (
  VOID
  )
/*++

Routine Description:

  Free the debug services.

Returns:

  EFI_SUCCESS   - Successful.
  Other value   - Something failed.

--*/
{
  EFI_STATUS  Status;

  //
  // Check the pre-conditions
  //
  if (mLogFile == NULL) {
    return EFI_SUCCESS;
  }

  //
  // Close the system log file
  //
  Status = gFT->TolProtocol->Close (
                               gFT->TolProtocol,
                               mLogFile
                               );
  if (EFI_ERROR(Status)) {
    return Status;
  }

  mLogFile = NULL;

  //
  // Done
  //
  return EFI_SUCCESS;
}


VOID
EfiSctDebug (
  IN UINTN              Level,
  IN CHAR16             *Format,
  ...
  )
/*++

Routine Description:

  Worker function for EFI_SCT_DEBUG marco.

--*/
{
  EFI_STATUS  Status;
  VA_LIST     Marker;
  CHAR16      Buffer[EFI_SCT_MAX_BUFFER_SIZE];
  CHAR16      *String;

  //
  // Check the debug masks
  //
  if ((Level & EFI_SCT_D_MASK) == 0) {
    return;
  }

  VA_START (Marker, Format);
  VSPrint (Buffer, EFI_SCT_MAX_BUFFER_SIZE, Format, Marker);
  VA_END (Marker);

  //
  // Deal with different error level
  //
  switch (Level) {
  case EFI_SCT_D_ERROR:
    String = PoolPrint (L"ERROR: %s\n", Buffer);
    break;

  case EFI_SCT_D_DEBUG:
    String = PoolPrint (L"DEBUG: %s\n", Buffer);
    break;

  case EFI_SCT_D_TRACE:
    String = PoolPrint (L"TRACE: %s\n", Buffer);
    break;

  default:
    return;
  }

  if (mLogFile != NULL) {
    //
    // Record into the system log file
    //
    Status = gFT->TolProtocol->Write (
                                 gFT->TolProtocol,
                                 mLogFile,
                                 String
                                 );
    if (EFI_ERROR(Status)) {
      BS->FreePool (String);
      return;
    }
  } else {
    //
    // Print the message into screen before calling InitializeDebugServices
    //
    Print (String);
  }

  BS->FreePool (String);
}
